import 'dart:async';

import 'package:flutter/material.dart';

import '../button/button_widget.dart';
import '../button/grident_button.dart';
import 'alert_const.dart';
import 'alert_config.dart';
import 'animation_transition.dart';

/// @author:   wzp_vicky
///  @time:     2023年9月14日08:21:08
///  @version:  V1.1
///  @description:

class VAlert {
  final String? id;
  final BuildContext context;
  final AlertType? type;
  final VAlertConfig style;
  final EdgeInsets? padding;
  final Widget? image;
  final String? title;
  final String? desc;
  final Widget content;
  final List<VButton>? buttons;
  final Function? closeFunction;
  final Widget? closeIcon;
  final bool onWillPopActive;
  final bool useRootNavigator;
  final AlertAnimation? alertAnimation;

  VAlert({
    required this.context,
    this.id,
    this.type,
    this.style = const VAlertConfig(),
    this.padding,
    this.image,
    this.title,
    this.desc,
    this.content = const SizedBox(),
    this.buttons,
    this.closeFunction,
    this.closeIcon,
    this.onWillPopActive = false,
    this.alertAnimation,
    this.useRootNavigator = true,
  });

  Future<bool?> show() async {
    return await showGeneralDialog(
        context: context,
        pageBuilder: (BuildContext buildContext, Animation<double> animation, Animation<double> secondaryAnimation) {
          return _buildDialog();
        },
        barrierDismissible: style.isOverlayTapDismiss,
        barrierLabel: MaterialLocalizations.of(context).modalBarrierDismissLabel,
        barrierColor: style.overlayColor,
        useRootNavigator: useRootNavigator,
        transitionDuration: style.animationDuration,
        transitionBuilder: (
          BuildContext context,
          Animation<double> animation,
          Animation<double> secondaryAnimation,
          Widget child,
        ) =>
            alertAnimation == null
                ? _showAnimation(animation, secondaryAnimation, child)
                : alertAnimation!(context, animation, secondaryAnimation, child));
  }

  Future<void> dismiss() async {
    Navigator.of(context, rootNavigator: useRootNavigator).pop();
  }

  Widget _buildDialog() {
    final Widget _child = Align(
      alignment: style.alertAlignment,
      child: ConstrainedBox(
        constraints: style.constraints ?? BoxConstraints.loose(Size.infinite),
        child: SingleChildScrollView(
          child: AlertDialog(
              key: id == null ? null : Key(id!),
              backgroundColor: style.backgroundColor ?? Theme.of(context).dialogBackgroundColor,
              shape: style.alertBorder ?? _defaultShape(),
              insetPadding: style.alertPadding,
              elevation: style.alertElevation,
              titlePadding: const EdgeInsets.all(0.0),
              title: Center(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    _getCloseButton(),
                    Padding(
                      padding: padding ?? EdgeInsets.fromLTRB(20, (style.isCloseButton ? 0 : 10), 20, 0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        children: <Widget>[
                          Center(
                            child: _getImage(),
                          ),
                          if (title != null)
                            _getTextWidget(
                              style.isTitleSelectable,
                              title,
                              style.titleStyle,
                              style.titleTextAlign,
                              style.titlePadding ??
                                  EdgeInsets.only(
                                    top: 15,
                                    bottom: desc == null ? 0 : 10,
                                  ),
                            ),
                          if (desc != null)
                            _getTextWidget(
                              style.isDescSelectable,
                              desc,
                              style.descStyle,
                              style.descTextAlign,
                              style.descPadding,
                            ),
                          content,
                        ],
                      ),
                    )
                  ],
                ),
              ),
              contentPadding: style.buttonAreaPadding,
              content: style.buttonsDirection == ButtonsDirection.row
                  ? Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      mainAxisSize: MainAxisSize.min,
                      children: _getButtons(),
                    )
                  : Column(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: _getButtons(),
                    )),
        ),
      ),
    );
    return onWillPopActive ? WillPopScope(onWillPop: () async => false, child: _child) : _child;
  }

  /// 返回 Text Widget or Selectable Text Widget with Padding
  /// based on isSelectable property.
  Widget _getTextWidget(bool isSelectable, String? text, TextStyle textStyle, TextAlign textAlign, EdgeInsets edgeInsets) {
    return Padding(
      padding: edgeInsets,
      child: _getTextWidgetBySelectable(
        isSelectable,
        text,
        textStyle,
        textAlign,
      ),
    );
  }

  /// Returns Text Widget or Selectable Text Widget
  /// based on isSelectable property.
  Widget _getTextWidgetBySelectable(bool isSelectable, String? text, TextStyle textStyle, TextAlign textAlign) {
    if (isSelectable) {
      return SelectableText(
        text ?? "",
        style: textStyle,
        textAlign: textAlign,
      );
    } else {
      return Text(
        text ?? "",
        style: textStyle,
        textAlign: textAlign,
      );
    }
  }

  /// 返回右上角的关闭按钮
  Widget _getCloseButton() {
    return style.isCloseButton
        ? Padding(
            padding: const EdgeInsets.fromLTRB(0, 10, 10, 0),
            child: GestureDetector(
              onTap: () {
                if (closeFunction == null) {
                  Navigator.of(context, rootNavigator: useRootNavigator).pop();
                } else {
                  closeFunction!();
                }
              },
              child: Container(
                alignment: FractionalOffset.topRight,
                child: closeIcon != null
                    ? Container(child: closeIcon)
                    : Container(
                        width: 20,
                        height: 20,
                        decoration: const BoxDecoration(
                          image: DecorationImage(
                            image: AssetImage(
                              '$kImagePath/close.png',
                              package: 'rflutter_alert',
                            ),
                          ),
                        ),
                      ),
              ),
            ),
          )
        : Container();
  }

  /// 返回按钮组.默认有取消按钮
  List<Widget> _getButtons() {
    List<Widget> expandedButtons = [];
    if (style.isButtonVisible) {
      if (buttons != null) {
        for (var button in buttons!) {
          var buttonWidget = Padding(
            padding: const EdgeInsets.only(left: 2, right: 2),
            child: button,
          );
          if ((button.width != null && buttons!.length == 1) || style.buttonsDirection == ButtonsDirection.column) {
            expandedButtons.add(buttonWidget);
          } else {
            expandedButtons.add(Expanded(
              child: buttonWidget,
            ));
          }
        }
      } else {
        Widget cancelButton = VButton(
          child: const Text(
            "CANCEL",
            style: TextStyle(color: Colors.white, fontSize: 20),
          ),

          onPressed: () => Navigator.pop(context),
        );
        if (style.buttonsDirection == ButtonsDirection.row) {
          cancelButton = Expanded(
            child: cancelButton,
          );
        }
        expandedButtons.add(cancelButton);
      }
    }
    return expandedButtons;
  }

  /// 返回 alert 边框样式
  ShapeBorder _defaultShape() {
    return RoundedRectangleBorder(
      borderRadius: BorderRadius.circular(10.0),
      side: const BorderSide(
        color: Colors.blueGrey,
      ),
    );
  }

  /// Returns alert image for icon
  Widget _getImage() {
    Widget response = image ?? Container();
    return response;
  }

  /// 弹出来时的动画
  _showAnimation(animation, secondaryAnimation, child) {
    switch (style.animationType) {
      case AnimationType.fromRight:
        return AnimationTransition.fromRight(animation, secondaryAnimation, child);
      case AnimationType.fromLeft:
        return AnimationTransition.fromLeft(animation, secondaryAnimation, child);
      case AnimationType.fromBottom:
        return AnimationTransition.fromBottom(animation, secondaryAnimation, child);
      case AnimationType.grow:
        return AnimationTransition.grow(animation, secondaryAnimation, child);
      case AnimationType.shrink:
        return AnimationTransition.shrink(animation, secondaryAnimation, child);
      case AnimationType.fromTop:
        return AnimationTransition.fromTop(animation, secondaryAnimation, child);
    }
  }
}
