import 'package:flutter/material.dart';

/// @author:   wzp_vicky
/// @time:     2022年10月27日23:58:00
/// @version:  V1.1
/// @description:自定义指示器，实现了2种：带有动画和不带动画，具体见下述
///
/// 我们看到官方提供一个UnderlineTabIndicator类，通过insets参数可以设置指示器的边距从而达到设置指示器宽度的效果，
/// 但是这并不能固定TabBar的宽度，而且当tabBar数量变化时或者文本长度改变，指示器宽度也会改变，
/// 我这里直接对UnderlineTabIndicator这个类进行了二次改造，
/// 关键代码：通过这个方法我们自定义返回已个矩形，自定义我们需要的宽度值即可。
/// */
/// 修改下划线自定义
class VTabIndicator extends Decoration {
  ///需要动画的时候，
  ///线的宽度：indicatorWidth
  ///线的高度：controlIndictorHeight
  ///下边距实实在在的来自indicatorBottom
  final TabController? tabController; //给指示器加的动画，如果传这个，高度和宽度就都不会使用borderSide
  final double controlIndicatorHeight;

  final double indicatorBottom; // 调整指示器下边距
  final double indicatorWidth; // 指示器宽度
  /// The color and weight of the horizontal line drawn below the selected tab.
  /// 不需要动画的时候，月就是tabController不传的时候
  /// 线的宽度：indicatorWidth
  /// 线的高度：borderSide.width
  /// 下边距实实在在的来自indicatorBottom
  final BorderSide borderSide; //建议不传control的时候传一下这

  const VTabIndicator({
    // 设置下标高度、颜色
    this.borderSide = const BorderSide(width: 10.0, color: Colors.blue),
    this.tabController,
    this.controlIndicatorHeight = 5,
    this.indicatorBottom = 0.0,
    required this.indicatorWidth,
  });

  @override
  BoxPainter createBoxPainter([VoidCallback? onChanged]) {
    return _UnderlinePainter(this, onChanged, tabController?.animation,
        indicatorWidth, controlIndicatorHeight);
  }

  //用来决定控制器宽度、位置
  Rect _indicatorRectFor(Rect indicator, TextDirection textDirection) {
    /// 自定义固定宽度
    double w = indicatorWidth;
    //中间坐标
    double centerWidth = (indicator.left + indicator.right) / 2;

    //这里是核心代码 //下划线靠左
    // return Rect.fromLTWH(indicator.left,
    //     indicator.bottom - borderSide.width, wantWidth, borderSide.width);

    return Rect.fromLTWH(
      //距离左边距
      tabController?.animation == null ? centerWidth - w / 2 : centerWidth - 1,
      // 距离上边距
      indicator.bottom - borderSide.width - indicatorBottom,
      w,
      borderSide.width,
    );
  }

  @override
  Path getClipPath(Rect rect, TextDirection textDirection) {
    return Path()..addRect(_indicatorRectFor(rect, textDirection));
  }
}

class _UnderlinePainter extends BoxPainter {
  Animation<double>? animation;
  double indicatorWidth;
  final double _controlIndicatorHeight;

  _UnderlinePainter(this.decoration, VoidCallback? onChanged, this.animation,
      this.indicatorWidth, this._controlIndicatorHeight)
      : super(onChanged);

  final VTabIndicator decoration;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration configuration) {
    assert(configuration.size != null);
    // 以offset坐标为左上角 size为宽高的矩形
    final Rect rect = offset & configuration.size!;
    final TextDirection textDirection = configuration.textDirection!;
    // 返回tab矩形
    final Rect indicator = decoration._indicatorRectFor(rect, textDirection)
      ..deflate(decoration.borderSide.width / 2.0);
    // 圆角画笔
    final Paint paint = decoration.borderSide.toPaint()
      ..style = PaintingStyle.fill
      ..strokeCap = StrokeCap.round;
    if (animation != null) {
      num x = animation!.value; // 变化速度 0-0.5-1-1.5-2...
      num d = x - x.truncate(); // 获取这个数字的小数部分
      num? y;
      if (d < 0.5) {
        y = 2 * d;
      } else if (d > 0.5) {
        y = 1 - 2 * (d - 0.5);
      } else {
        y = 1;
      }
      canvas.drawRRect(
          RRect.fromRectXY(
              Rect.fromCenter(
                  center: indicator.centerLeft,
                  // 这里控制最长为多长
                  width: indicatorWidth * 0.5 * y + indicatorWidth,
                  height: _controlIndicatorHeight),
              // 圆角
              30,
              30),
          paint);
    } else {
      canvas.drawLine(indicator.bottomLeft, indicator.bottomRight, paint);
    }
  }
}
